/**
 * PaymentDisplay for Stripe
 *
 * @author Patrick Müller (www.pcsg.de)
 */
define('package/quiqqer/payment-stripe/bin/controls/PaymentDisplay', [

    'qui/controls/Control',
    'qui/controls/buttons/Button',
    'qui/controls/loader/Loader',

    'utils/Controls',

    'Ajax',
    'Locale',
    'package/quiqqer/payment-stripe/bin/Stripe',

    'css!package/quiqqer/payment-stripe/bin/controls/PaymentDisplay.css'

], function (QUIControl, QUIButton, QUILoader, QUIControlUtils, QUIAjax, QUILocale, StripeHandler) {
    "use strict";

    var pkg = 'quiqqer/payment-stripe';

    return new Class({

        Extends: QUIControl,
        Type   : 'package/quiqqer/payment-stripe/bin/controls/PaymentDisplay',

        Binds: [
            '$onImport',
            '$loadStripeJs',
            '$showStripeForm',
            '$onPaymentFormSubmit',
            '$handlePaymentIntentConfirmation',
            '$handleAdditionalCustomerAction',
            '$onStripeError',
            '$showErrorMsg',
            '$showMsg',
            '$onGeneralError',
            '$getPayBtn',
            '$onBrowserPaySubmit'
        ],

        options: {
            orderhash    : '',
            successful   : false,
            paymentmethod: 'card',
            country      : '',
            currency     : '',
            amount       : 0
        },

        initialize: function (options) {
            this.parent(options);

            this.$Stripe            = null;
            this.$Elements          = null;
            this.$StripeFormElement = null;

            this.$PayBtn          = null;
            this.$MsgElm          = null;
            this.$OrderProcess    = null;
            this.Loader           = new QUILoader();
            this.$BrowserPayEvent = null;

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * Event: onImport
         */
        $onImport: function () {
            var self = this;
            var Elm  = this.getElm();

            if (Elm.getElement('.message-error')) {
                (function () {
                    self.fireEvent('processingError', [self]);
                }).delay(2000);
            }

            if (!Elm.getElement('.quiqqer-payment-stripe-content')) {
                return;
            }

            this.Loader.inject(Elm);
            this.$MsgElm = Elm.getElement('.quiqqer-payment-stripe-message');

            QUIControlUtils.getControlByElement(
                Elm.getParent('[data-qui="package/quiqqer/order/bin/frontend/controls/OrderProcess"]')
            ).then(function (OrderProcess) {
                self.$OrderProcess = OrderProcess;

                if (self.getAttribute('successful')) {
                    OrderProcess.next();
                    return;
                }

                self.$loadStripeJs();
            });
        },

        /**
         * Load Stripe.js
         */
        $loadStripeJs: function () {
            var self = this;

            var onStripeJsLoaded = function () {
                self.$Stripe   = Stripe(self.getAttribute('publickey'));
                self.$Elements = self.$Stripe.elements();

                self.$showStripeForm();
            };

            if (typeof Stripe !== 'undefined') {
                onStripeJsLoaded();
                return;
            }

            this.$OrderProcess.Loader.show();

            new Element('script', {
                async: "async",
                src  : "https://js.stripe.com/v3/"
            }).inject(document.body);

            var checkStripeLoaded = setInterval(function () {
                if (typeof Stripe === 'undefined') {
                    return;
                }

                clearInterval(checkStripeLoaded);

                onStripeJsLoaded();
            }, 200);
        },

        /**
         * Show Stripe payment method form
         */
        $showStripeForm: function () {
            var self = this;

            this.$OrderProcess.Loader.show();

            var PayBtnElm   = this.$Elm.getElement('.quiqqer-payment-stripe-submit'),
                PaymentForm = this.$Elm.getElement('.quiqqer-payment-stripe-form');

            switch (this.getAttribute('paymentmethod')) {
                case 'browser_pay':
                    var PaymentRequest = self.$Stripe.paymentRequest({
                        country : this.getAttribute('country'),
                        currency: this.getAttribute('currency'),
                        total   : {
                            label : QUILocale.get(pkg, 'controls.PaymentDisplay.browser_pay.order_label', {
                                orderHash: this.getAttribute('orderhash')
                            }),
                            amount: this.getAttribute('amount')
                        }
                        //requestPayerName: true,
                        //requestPayerEmail: true,
                    });

                    this.$StripeFormElement = this.$Elements.create('paymentRequestButton', {
                        paymentRequest: PaymentRequest,
                        style         : {
                            base   : {
                                color          : '#13151b',
                                fontSize       : '16px',
                                fontFamily     : '"Open Sans", sans-serif',
                                fontSmoothing  : 'antialiased',
                                '::placeholder': {
                                    color: '#9ba2aa'
                                }
                            },
                            invalid: {
                                color   : '#e5424d',
                                ':focus': {
                                    color: '#303238'
                                }
                            }
                        }
                    });

                    PaymentRequest.canMakePayment().then(function (canMakePayment) {
                        if (!canMakePayment) {
                            self.$showErrorMsg(
                                QUILocale.get(pkg, 'controls.PaymentDisplay.browser_pay.error.cannot_use')
                            );

                            self.fireEvent('processingError', [self]);

                            PaymentForm.destroy();
                            return;
                        }

                        PaymentRequest.on('paymentmethod', self.$onBrowserPaySubmit);

                        self.$StripeFormElement.mount(PaymentForm);
                        self.$OrderProcess.Loader.hide();
                    });

                    break;

                case 'card':
                default:
                    //this.$showMsg(QUILocale.get(pkg, 'PaymentDisplay.card.info'));

                    this.$PayBtn = this.$getPayBtn().inject(PayBtnElm);

                    this.$StripeFormElement = this.$Elements.create('card', {
                        style: {
                            base   : {
                                color          : '#13151b',
                                fontSize       : '16px',
                                fontFamily     : '"Open Sans", sans-serif',
                                fontSmoothing  : 'antialiased',
                                '::placeholder': {
                                    color: '#9ba2aa'
                                }
                            },
                            invalid: {
                                color   : '#e5424d',
                                ':focus': {
                                    color: '#303238'
                                }
                            }
                        }
                    });

                    this.$StripeFormElement.mount(PaymentForm);
                    this.$OrderProcess.Loader.hide();
                    break;
            }
        },

        /**
         * Handle special Stripe paymentRequest submit (Browser Pay)
         *
         * @param {Object} event
         */
        $onBrowserPaySubmit: function (event) {
            var self = this;

            this.$BrowserPayEvent = event;

            StripeHandler.createPaymentIntent(
                this.getAttribute('orderhash'),
                event.paymentMethod.id
            ).then(this.$handlePaymentIntentConfirmation, function () {
                self.$BrowserPayEvent.complete('fail');
                self.$onGeneralError();
            });
        },

        /**
         * Handle payment form submit
         */
        $onPaymentFormSubmit: function () {
            var self = this;

            this.$PayBtn.disable();
            this.$OrderProcess.Loader.show(QUILocale.get(pkg, 'PaymentDisplay.checkout_process'));

            this.$Stripe.createPaymentMethod(
                this.getAttribute('paymentmethod'),
                this.$StripeFormElement
            ).then(function (result) {
                if (result.error) {
                    self.$onStripeError(result.error);
                    return;
                }

                StripeHandler.createPaymentIntent(
                    self.getAttribute('orderhash'),
                    result.paymentMethod.id
                ).then(self.$handlePaymentIntentConfirmation, self.$onGeneralError);
            }, this.$onGeneralError);
        },

        /**
         * Handle Stripe PaymentIntent confirmation result
         *
         * @param {Object} Confirmation
         */
        $handlePaymentIntentConfirmation: function (Confirmation) {
            switch (Confirmation.status) {
                case 'success':
                case 'active':  // subscriptions
                    if (this.$BrowserPayEvent) {
                        this.$BrowserPayEvent.complete('success');
                    }

                    this.$OrderProcess.next();
                    break;

                case 'action_required':
                    this.$handleAdditionalCustomerAction(Confirmation);
                    break;

                case 'retry_payment_method':
                    if (this.$BrowserPayEvent) {
                        this.$BrowserPayEvent.complete('fail');
                    }

                    this.$OrderProcess.Loader.hide();
                    this.$PayBtn.enable();
                    break;

                case 'api_error':
                    if (this.$BrowserPayEvent) {
                        this.$BrowserPayEvent.complete('fail');
                    }

                    this.$OrderProcess.next();
                    break;

                default:
                    if (this.$BrowserPayEvent) {
                        this.$BrowserPayEvent.complete('fail');
                    }

                    this.$showErrorMsg(
                        QUILocale.get(pkg, 'PaymentDisplay.validation_error')
                    );

                    this.fireEvent('processingError', [this]);
                    this.$OrderProcess.resize();
                    this.$OrderProcess.Loader.hide();

                    this.$PayBtn.enable();
            }
        },

        /**
         * Handle additional customer action (only applies in some cases; e.g. 3D-Secure)
         *
         * @param {Object} Confirmation - Confirmation data from PaymentIntent
         */
        $handleAdditionalCustomerAction: function (Confirmation) {
            var self = this;

            this.$OrderProcess.Loader.show();
            this.$Stripe.handleCardAction(Confirmation.clientSecret).then(function (result) {
                if (result.error) {
                    self.$onStripeError(result.error);
                    return;
                }

                StripeHandler.confirmPaymentIntent(
                    self.getAttribute('orderhash')
                ).then(self.$handlePaymentIntentConfirmation);
            });
        },

        /**
         * Handle Stripe API error
         *
         * @param {Object} Error
         */
        $onStripeError: function (Error) {
            this.$showErrorMsg(Error.message);
            this.fireEvent('processingError', [this]);

            if (this.$PayBtn) {
                this.$PayBtn.enable();
            }

            this.$OrderProcess.Loader.hide();
        },

        /**
         * Displays a general error message
         */
        $onGeneralError: function () {
            if (this.$PayBtn) {
                this.$PayBtn.enable();
            }

            this.$showErrorMsg(QUILocale.get(pkg, 'PaymentDisplay.error.general'));
            this.fireEvent('processingError', [this]);
        },

        /**
         * Show error msg
         *
         * @param {String} msg
         */
        $showErrorMsg: function (msg) {
            this.$MsgElm.set(
                'html',
                '<p class="message-error">' + msg + '</p>'
            );
        },

        /**
         * Show normal msg
         *
         * @param {String} msg
         */
        $showMsg: function (msg) {
            this.$MsgElm.set(
                'html',
                '<p>' + msg + '</p>'
            );
        },

        /**
         * Get payment button
         *
         * @return {QUIButton}
         */
        $getPayBtn: function () {
            var PayBtnElm = this.$Elm.getElement('.quiqqer-payment-stripe-submit');

            switch (this.getAttribute('paymentmethod')) {
                case 'card':
                default:
                    return new QUIButton({
                        'class'  : 'btn-primary button',
                        text     : QUILocale.get(pkg, 'PaymentDisplay.btn_pay.card.text', {
                            display_price: PayBtnElm.get('data-price')
                        }),
                        alt      : QUILocale.get(pkg, 'PaymentDisplay.btn_pay.card.title', {
                            display_price: PayBtnElm.get('data-price')
                        }),
                        title    : QUILocale.get(pkg, 'PaymentDisplay.btn_pay.card.title', {
                            display_price: PayBtnElm.get('data-price')
                        }),
                        textimage: 'fa fa-credit-card',
                        events   : {
                            onClick: this.$onPaymentFormSubmit
                        }
                    });
            }
        }
    });
});