/**
 * PaymentDisplay for Stripe (recurring payments)
 *
 * @author Patrick Müller (www.pcsg.de)
 */
define('package/quiqqer/payment-stripe/bin/controls/Recurring/PaymentDisplay', [

    'package/quiqqer/payment-stripe/bin/controls/PaymentDisplay',
    'package/quiqqer/payment-stripe/bin/Stripe',

    'qui/controls/buttons/Button',

    'Locale',

    'css!package/quiqqer/payment-stripe/bin/controls/Recurring/PaymentDisplay.css'

], function (BasePaymentDisplay, StripeHandler, QUIButton, QUILocale) {
    "use strict";

    var pkg = 'quiqqer/payment-stripe';

    return new Class({

        Extends: BasePaymentDisplay,
        Type   : 'package/quiqqer/payment-stripe/bin/controls/Recurring/PaymentDisplay',

        Binds: [
            '$onPaymentFormSubmit',
            '$getPayBtn',
            '$handleSubscriptionConfirmation'
        ],

        initialize: function (options) {
            this.parent(options);
            this.$resetPaymentMethod = false;
        },

        /**
         * Submit payment form
         */
        $onPaymentFormSubmit: function () {
            var self = this;

            this.$PayBtn.disable();
            this.$OrderProcess.Loader.show(QUILocale.get(pkg, 'PaymentDisplay.checkout_process'));

            this.$Stripe.createPaymentMethod(
                this.getAttribute('paymentmethod'),
                this.$StripeFormElement
            ).then(function (result) {
                if (result.error) {
                    self.$onStripeError(result.error);
                    return;
                }

                StripeHandler.createSubscription(
                    self.getAttribute('orderhash'),
                    result.paymentMethod.id,
                    self.$resetPaymentMethod
                ).then(self.$handleSubscriptionConfirmation, self.$onGeneralError);
            }, this.$onGeneralError);
        },

        /**
         * Handle special Stripe paymentRequest submit (Browser Pay)
         *
         * @param {Object} event
         */
        $onBrowserPaySubmit: function (event) {
            var self = this;

            this.$BrowserPayEvent = event;

            StripeHandler.createSubscription(
                self.getAttribute('orderhash'),
                event.paymentMethod.id,
                self.$resetPaymentMethod
            ).then(self.$handleSubscriptionConfirmation, function () {
                self.$BrowserPayEvent.complete('fail');
                self.$onGeneralError();
            });
        },

        /**
         * Handle additional customer action (only applies in some cases; e.g. 3D-Secure)
         *
         * @param {Object} Confirmation - Confirmation data from PaymentIntent
         */
        $handleAdditionalCustomerAction: function (Confirmation) {
            var self = this;

            this.$OrderProcess.Loader.show();
            this.$Stripe.handleCardPayment(Confirmation.clientSecret).then(function (result) {
                if (result.error) {
                    self.$onStripeError(result.error);
                    return;
                }

                StripeHandler.confirmSubscription(
                    self.getAttribute('orderhash')
                ).then(self.$handleSubscriptionConfirmation);
            });
        },

        /**
         * Handle Stripe PaymentIntent confirmation result
         *
         * @param {Object} Confirmation
         */
        $handleSubscriptionConfirmation: function (Confirmation) {
            switch (Confirmation.status) {
                case 'active':
                    if (this.$BrowserPayEvent) {
                        this.$BrowserPayEvent.complete('success');
                    }

                    this.$OrderProcess.next();
                    break;

                case 'action_required':
                    this.$handleAdditionalCustomerAction(Confirmation);
                    break;

                case 'retry_payment_method':
                    if (this.$BrowserPayEvent) {
                        this.$BrowserPayEvent.complete('fail');
                    }

                    this.$OrderProcess.Loader.hide();

                    this.$showErrorMsg(
                        QUILocale.get(pkg, 'controls.Recurring.PaymentDisplay.subscripton_payment_not_confirmed')
                    );

                    this.$resetPaymentMethod = true;
                    this.$PayBtn.enable();
                    break;

                case 'api_error':
                    if (this.$BrowserPayEvent) {
                        this.$BrowserPayEvent.complete('fail');
                    }

                    this.$OrderProcess.next();
                    break;

                default:
                    if (this.$BrowserPayEvent) {
                        this.$BrowserPayEvent.complete('fail');
                    }

                    this.$showErrorMsg(
                        QUILocale.get(pkg, 'PaymentDisplay.validation_error')
                    );

                    this.fireEvent('processingError', [this]);
                    this.$OrderProcess.resize();
                    this.$OrderProcess.Loader.hide();

                    this.$PayBtn.enable();
            }
        },

        /**
         * Get payment button
         *
         * @return {QUIButton}
         */
        $getPayBtn: function () {
            switch (this.getAttribute('paymentmethod')) {
                case 'card':
                default:
                    return new QUIButton({
                        'class'  : 'btn-primary button',
                        text     : QUILocale.get(pkg, 'PaymentDisplay.Recurring.btn_pay.text'),
                        alt      : QUILocale.get(pkg, 'PaymentDisplay.Recurring.btn_pay.title'),
                        title    : QUILocale.get(pkg, 'PaymentDisplay.Recurring.btn_pay.title'),
                        textimage: 'fa fa-check-square-o',
                        events   : {
                            onClick: this.$onPaymentFormSubmit
                        }
                    });
            }
        }
    });
});