/**
 * List of all Stripe Billing Plans
 *
 * @module package/quiqqer/payment-stripe/bin/controls/backend/settings/BillingPlans
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/payment-stripe/bin/controls/backend/settings/BillingPlans', [

    'qui/controls/Control',
    'qui/controls/loader/Loader',
    'qui/controls/windows/Confirm',
    'qui/controls/buttons/Button',
    'controls/grid/Grid',
    'package/quiqqer/payment-stripe/bin/Stripe',

    'Locale',

    'css!package/quiqqer/payment-stripe/bin/controls/backend/settings/BillingPlans.css'

], function (QUIControl, QUILoader, QUIConfirm, QUIButton, Grid, Stripe, QUILocale) {
    "use strict";

    var lg = 'quiqqer/payment-stripe';

    return new Class({

        Extends: QUIControl,
        Type   : 'package/quiqqer/payment-stripe/bin/controls/backend/settings/BillingPlans',

        Binds: [
            'refresh',
            '$onCreate',
            '$onImport',
            '$onResize',
            '$clickDelete'
        ],

        initialize: function (options) {
            this.parent(options);

            this.$Grid    = null;
            this.$Content = null;
            this.Loader   = new QUILoader();
            this.$startId = null;

            this.addEvents({
                onCreate: this.$onCreate,
                onResize: this.$onResize,
                onImport: this.$onImport
            });
        },

        /**
         * Refresh the grid
         */
        refresh: function () {
            if (!this.$Grid) {
                return;
            }

            this.Loader.show();

            return Stripe.getBillingPlans({
                perPage: this.$Grid.options.perPage,
                page   : this.$Grid.options.page,
                sortBy : this.$Grid.options.sortBy,
                sortOn : this.$Grid.options.sortOn
            }).then(function (result) {
                var TableButtons = this.$Grid.getAttribute('buttons');
                TableButtons.delete.disable();

                var detailsUrl,
                    detailsTitle = QUILocale.get(lg, 'controls.backend.settings.BillingPlans.open_in_dashboard');

                if (result.data.length && result.data[0].isSandbox) {
                    detailsUrl = 'https://dashboard.stripe.com/test/plans/';
                } else {
                    detailsUrl = 'https://dashboard.stripe.com/plans/'
                }

                var $onDetailsClick = function (event) {
                    var Link = new Element('a', {
                        href  : detailsUrl + event.target.get('data-id'),
                        target: '_blank'
                    }).inject(document.body);

                    Link.click();
                    Link.destroy();
                };

                for (var i = 0, len = result.data.length; i < len; i++) {
                    var Row = result.data[i];

                    Row.details = new Element('span', {
                        'class'  : 'fa fa-cc-stripe quiqqer-payment-stripe-plans',
                        title    : detailsTitle,
                        'data-id': Row.id,
                        events   : {
                            click: $onDetailsClick
                        }
                    });
                }

                this.$Grid.setData(result);
                this.Loader.hide();
            }.bind(this), function () {
                this.destroy();
            }.bind(this));
        },

        /**
         * Event Handling
         */

        $onImport: function () {
            this.$Content = new Element('div', {
                'class': 'quiqqer-payment-stripe-plans field-container-field'
            }).inject(this.getElm(), 'after');

            this.Loader.inject(this.$Content);

            this.$Content.getParent('form').setStyle('height', '100%');
            this.$Content.getParent('table').setStyle('height', '100%');
            this.$Content.getParent('tbody').setStyle('height', '100%');
            this.$Content.getParent('.field-container').setStyle('height', '100%');

            this.create();
            this.$onCreate();
            this.refresh();
        },

        /**
         * event : on panel create
         */
        $onCreate: function () {
            var self = this;

            // Buttons

            // Grid
            var Container = new Element('div', {
                style: {
                    height: '100%',
                    width : '100%'
                }
            }).inject(this.$Content);

            this.$Grid = new Grid(Container, {
                pagination       : true,
                multipleSelection: true,
                serverSort       : true,

                accordion           : false,
                autoSectionToggle   : false,
                openAccordionOnClick: false,

                buttons    : [{
                    name     : 'delete',
                    text     : QUILocale.get(lg, 'controls.backend.settings.BillingPlans.tbl.btn.delete'),
                    textimage: 'fa fa-trash',
                    events   : {
                        onClick: this.$clickDelete
                    }
                }],
                columnModel: [{
                    header   : QUILocale.get(lg, 'controls.backend.settings.BillingPlans.tbl.details'),
                    dataIndex: 'details',
                    dataType : 'node',
                    width    : 50
                }, {
                    header   : QUILocale.get(lg, 'controls.backend.settings.BillingPlans.tbl.id'),
                    dataIndex: 'id',
                    dataType : 'string',
                    width    : 200
                }, {
                    header   : QUILocale.get(lg, 'controls.backend.settings.BillingPlans.tbl.nickname'),
                    dataIndex: 'nickname',
                    dataType : 'string',
                    width    : 250
                }, {
                    header   : QUILocale.get(lg, 'controls.backend.settings.BillingPlans.tbl.created_at'),
                    dataIndex: 'created',
                    dataType : 'string',
                    width    : 150
                }]
            });

            this.$Grid.addEvents({
                onRefresh: this.refresh,

                onClick: function () {
                    var TableButtons = self.$Grid.getAttribute('buttons'),
                        selected     = self.$Grid.getSelectedData().length;

                    if (!Object.getLength(TableButtons)) {
                        return;
                    }

                    if (selected === 1) {
                        TableButtons.delete.enable();
                    } else {
                        TableButtons.delete.disable();
                    }
                }
            });

            this.$onResize();
        },

        /**
         * event : on panel resize
         */
        $onResize: function () {
            if (!this.$Grid) {
                return;
            }

            var size = this.$Content.getSize();

            this.$Grid.setHeight(size.y - 20);
            this.$Grid.setWidth(size.x - 20);
            this.$Grid.resize();
        },

        /**
         * Delete Billing Plan dialog
         */
        $clickDelete: function () {
            var self   = this;
            var planId = this.$Grid.getSelectedData()[0].id;

            new QUIConfirm({
                maxHeight: 450,
                maxWidth : 650,
                autoclose: false,

                information: QUILocale.get(lg, 'controls.backend.settings.BillingPlans.delete.information'),
                title      : QUILocale.get(lg, 'controls.backend.settings.BillingPlans.delete.title', {
                    planId: planId
                }),
                texticon   : 'fa fa-trash',
                text       : QUILocale.get(lg, 'controls.backend.settings.BillingPlans.delete.text', {
                    planId: planId
                }),
                icon       : 'fa fa-trash',
                ok_button  : {
                    text     : QUILocale.get(lg, 'controls.backend.settings.BillingPlans.delete.ok'),
                    textimage: 'icon-ok fa fa-trash'
                },
                events     : {
                    onOpen  : function (Win) {
                        Win.getButton('submit').getElm().addClass('btn-red');
                    },
                    onSubmit: function (Popup) {
                        Popup.Loader.show();

                        Stripe.deleteOffer(planId).then(function () {
                            self.refresh();
                            Popup.close();
                        }, function () {
                            Popup.Loader.hide();
                        })
                    }
                }

            }).open();
        }
    });
});
