<?php

namespace QUI\ERP\Payments\Stripe;

use QUI;
use QUI\ERP\Accounting\Payments\Api\AbstractPaymentProvider;
use QUI\ERP\Accounting\Payments\Types\Factory;
use QUI\ERP\Payments\Stripe\PaymentMethods\Card;
use Stripe\Stripe as StripeApiClient;
use QUI\ERP\Payments\Stripe\PaymentMethods\Recurring\Card as CardRecurring;
use QUI\ERP\Payments\Stripe\PaymentMethods\ApplePay;
use QUI\ERP\Payments\Stripe\PaymentMethods\GooglePay;
use QUI\ERP\Payments\Stripe\PaymentMethods\MicrosoftPay;
use QUI\ERP\Payments\Stripe\PaymentMethods\Recurring\ApplePay as ApplePayRecurring;
use QUI\ERP\Payments\Stripe\PaymentMethods\Recurring\GooglePay as GooglePayRecurring;
use QUI\ERP\Payments\Stripe\PaymentMethods\Recurring\MicrosoftPay as MicrosoftPayRecurring;
use QUI\System\VhostManager;

/**
 * Class Provider
 *
 * PaymentProvider class for PayPal
 */
class Provider extends AbstractPaymentProvider
{
    /**
     * Database tables
     */
    const TBL_STRIPE_BILLING_PLANS                      = 'stripe_billing_plans';
    const TBL_STRIPE_BILLING_SUBSCRIPTIONS              = 'stripe_billing_subscriptions';
    const TBL_STRIPE_BILLING_SUBSCRIPTIONS_TRANSACTIONS = 'stripe_billing_subscriptions_transactions';

    /**
     * @return array
     */
    public function getPaymentTypes()
    {
        return \array_merge([
            Card::class,
            GooglePay::class,
            ApplePay::class,
            MicrosoftPay::class
        ], self::getRecurringPaymentTypes());
    }

    /**
     * @return array
     */
    public static function getRecurringPaymentTypes()
    {
        return [
            CardRecurring::class,
            ApplePayRecurring::class,
            GooglePayRecurring::class,
            MicrosoftPayRecurring::class
        ];
    }

    /**
     * Get API setting
     *
     * @param string $setting - Setting name
     * @return string|number|false
     */
    public static function getApiSetting($setting)
    {
        try {
            $Conf = QUI::getPackage('quiqqer/payment-stripe')->getConfig();
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            return false;
        }

        return $Conf->get('api', $setting);
    }

    /**
     * Get Stripe public key ("Publishable key")
     *
     * @return bool
     */
    public static function getPublicKey()
    {
        if (self::isSandbox()) {
            return self::getApiSetting('sandbox_public_key');
        }

        return self::getApiSetting('public_key');
    }

    /**
     * Get Stripe client secret
     *
     * @param bool $considerSandbox (optional) - Considers sandbox settings when retrieving key [default: true]
     * @return bool
     */
    public static function getClientSecret($considerSandbox = true)
    {
        if ($considerSandbox && self::isSandbox()) {
            return self::getApiSetting('sandbox_client_secret');
        }

        return self::getApiSetting('client_secret');
    }

    /**
     * Check if current Stripe API instance runs in sandbox mode
     *
     * @return bool
     */
    public static function isSandbox()
    {
        return self::getApiSetting('sandbox') ? true : false;
    }

    /**
     * Check if the PayPal API settings are correct
     *
     * @return bool
     */
    public static function isApiSetUp()
    {
        try {
            $Conf        = QUI::getPackage('quiqqer/payment-stripe')->getConfig();
            $apiSettings = $Conf->getSection('api');
        } catch (QUI\Exception $Exception) {
            return false;
        }

        $isSetup = true;

        if ($apiSettings['sandbox']) {
            if (empty($apiSettings['sandbox_public_key']) || empty($apiSettings['sandbox_public_key'])) {
                $isSetup = false;
            }
        } else {
            if (empty($apiSettings['public_key']) || empty($apiSettings['client_secret'])) {
                $isSetup = false;
            }
        }

        if (!$isSetup) {
            // @todo URL anpassen
            QUI\System\Log::addError(
                'Your Stripe API credentials seem to be (partially) missing.'
                .' Stripe CAN NOT be used at the moment. Please enter all your'
                .' API credentials. See https://dev.quiqqer.com/quiqqer/payment-stripe/wikis/api-configuration'
                .' for further instructions.'
            );

            return false;
        }

        return true;
    }

    /**
     * Sets up the Stripe API Client for usage
     *
     * @return void
     */
    public static function setupApi()
    {
        StripeApiClient::setApiKey(self::getClientSecret());
    }

    /**
     * @return string
     */
    public static function getStripeBillingPlansTable()
    {
        return QUI::getDBTableName(self::TBL_STRIPE_BILLING_PLANS);
    }

    /**
     * @return string
     */
    public static function getStripeBillingSubscriptionsTable()
    {
        return QUI::getDBTableName(self::TBL_STRIPE_BILLING_SUBSCRIPTIONS);
    }

    /**
     * @return string
     */
    public static function getStripeBillingSubscriptionsTransactionsTable()
    {
        return QUI::getDBTableName(self::TBL_STRIPE_BILLING_SUBSCRIPTIONS_TRANSACTIONS);
    }

    /**
     * quiqqer/quiqqer: onRequest
     *
     * Handle registration request for Apple Pay
     *
     * @param QUI\Rewrite $Rewrite
     * @param String $url
     */
    public static function onApplePayRegisterRequest(QUI\Rewrite $Rewrite, $url)
    {
        if ($url !== '.well-known/apple-developer-merchantid-domain-association') {
            return;
        }

        try {
            $VhostManager = new VhostManager();
            $vhosts       = $VhostManager->getList();
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return;
        }

        $headers = getallheaders();

        if (empty($headers['Host'])) {
            exit;
        }

        try {
            $assocFileContent = \file_get_contents(self::getApplePayDomainAssociationFile());
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            exit;
        }

        $Response = QUI::getGlobalResponse();

        $Response->setStatusCode(200);
        $Response->setContent($assocFileContent);

        foreach ($vhosts as $domain => $data) {
            if ($headers['Host'] === $domain) {
                $Response->send();
                exit;
            }
        }

        exit;
    }

    /**
     * @throws QUI\Exception
     */
    public static function getApplePayDomainAssociationFile()
    {
        $varDir    = QUI::getPackage('quiqqer/payment-stripe')->getVarDir();
        $assocFile = $varDir.'apple-developer-merchantid-domain-association';

        if (!\file_exists($assocFile)) {
            self::updateApplePayDomainAssociationFile();
        }

        return $assocFile;
    }

    /**
     * Update the domain association file for Apple Pay domain registration
     *
     * @throws QUI\Exception
     */
    public static function updateApplePayDomainAssociationFile()
    {
        $varDir    = QUI::getPackage('quiqqer/payment-stripe')->getVarDir();
        $assocFile = $varDir.'apple-developer-merchantid-domain-association';

        \file_put_contents(
            $assocFile,
            \file_get_contents('https://stripe.com/files/apple-pay/apple-developer-merchantid-domain-association')
        );
    }

    /**
     * Setup all default payment types
     *
     * @throws QUI\Database\Exception
     * @throws QUI\ERP\Accounting\Payments\Exception
     * @throws QUI\Exception
     */
    public static function setupDefaultPaymentMethods()
    {
        // create the standard payment types
        $Locale   = new QUI\Locale();
        $Factory  = new Factory();
        $children = $Factory->getChildren();

        $existingTypes = \array_map(function ($PaymentType) {
            /* @var $PaymentType QUI\ERP\Accounting\Payments\Types\Payment */
            return $PaymentType->getAttribute('payment_type');
        }, $children);

        $existingTypes = \array_flip($existingTypes);
        $createTypes   = \array_merge([
            Card::class,
            GooglePay::class,
            ApplePay::class,
            MicrosoftPay::class
        ], self::getRecurringPaymentTypes());

        $langs = QUI::availableLanguages();

        foreach ($createTypes as $paymentTypeClass) {
            if (isset($existingTypes[$paymentTypeClass])) {
                continue;
            }

            /** @var AbstractBasePayment $PaymentType */
            $PaymentType = new $paymentTypeClass();

            $Payment = $Factory->createChild([
                'payment_type' => $paymentTypeClass
            ]);

            // Set title, description and working title for all languages
            $title        = [];
            $description  = [];
            $workingTitle = [];

            foreach ($langs as $lang) {
                $Locale->setCurrent($lang);
                $PaymentType->setLocale($Locale);

                $title[$lang]        = $PaymentType->getFrontendTitle();
                $description[$lang]  = $PaymentType->getFrontendDescription();
                $workingTitle[$lang] = $PaymentType->getTitle();
            }

            $Payment->setTitle($title);
            $Payment->setDescription($description);
            $Payment->setWorkingTitle($workingTitle);
        }
    }
}
